import { memo, useRef, useState, useMemo, useEffect } from 'react';
import { Handle, Position } from 'reactflow';
import { NodeAttribute, RelationNodeAttributes, SchemaReactflowRelationNode, toHandleId } from '../../../model';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useConfig,
  useQuerybuilderGraph,
  useQuerybuilderSettings,
} from '@graphpolaris/shared/lib/data-access';
import { addWarning } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { setQuerybuilderGraphology, toQuerybuilderGraphology } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import graphology from 'graphology';
import { LeftHandle, RelationshipHandleArrowType, RightHandle } from './relation-handles';
import { PillDropdown } from '../../pilldropdown/pilldropdown';

/**
 * Component to render a relation flow element
 * @param { FlowElement<RelationData>} param0 The data of a relation flow element.
 */
export const RelationPill = memo((node: SchemaReactflowRelationNode) => {
  const data = node.data;
  const graph = useQuerybuilderGraph();
  const graphologyGraph = useMemo(() => toQuerybuilderGraphology(graph), [graph]);
  const settings = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const graphologyNodeAttributes = useMemo<RelationNodeAttributes | undefined>(
    () => (graphologyGraph.hasNode(node.id) ? { ...(graphologyGraph.getNodeAttributes(node.id) as RelationNodeAttributes) } : undefined),
    [node.id]
  );
  const attributeEdges = useMemo(
    () => graph.edges.filter((edge) => edge.source === node.id && !!edge?.attributes?.sourceHandleData.attributeType),
    [graph]
  );
  const [hovered, setHovered] = useState(false);
  const [handleBeingDragged, setHandleBeingDragged] = useState(-1);

  const [depth, setDepth] = useState<{ min: number; max: number }>({
    min: data.depth.min || settings.depth.min,
    max: data.depth.max || settings.depth.max,
  });

  const [direction, setDirection] = useState<RelationshipHandleArrowType>('right');

  useEffect(() => {
    setDepth({ min: data.depth.min || settings.depth.min, max: data.depth.max || settings.depth.max });
  }, [data.depth]);

  const onNodeUpdated = () => {
    if (depth.min < 0) {
      dispatch(addWarning('The minimum depth cannot be smaller than 0'));
    } else if (depth.max > 99) {
      dispatch(addWarning('The maximum depth cannot be larger than 99'));
    } else if (depth.min > depth.max) {
      dispatch(addWarning('The minimum depth cannot be larger than the maximum depth'));
    } else {
      graphologyGraph.setNodeAttribute<any>(node.id, 'depth', depth);
      dispatch(setQuerybuilderGraphology(graphologyGraph));
    }
  };

  const onChangeDirection = () => {
    if (direction === 'right') {
      setDirection('both');
      graphologyGraph.setNodeAttribute<any>(node.id, 'direction', 'both');
    } else {
      setDirection('right');
      graphologyGraph.setNodeAttribute<any>(node.id, 'direction', 'right');
    }
    dispatch(setQuerybuilderGraphology(graphologyGraph));
  };

  const calcWidth = (data: number) => {
    return data.toString().length + 0.5 + 'ch';
  };

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) setHovered(true);
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    if (hovered) setHovered(false);
  };

  const onHandleMouseDown = (attribute: NodeAttribute, i: number, event: React.MouseEvent) => {
    setHandleBeingDragged(i);
    window.addEventListener('mouseup', onHandleMouseUp, true);
  };

  const onHandleMouseUp = () => {
    setHandleBeingDragged(-1);
    window.removeEventListener('mouseup', onHandleMouseUp, true);
  };

  return (
    <div
      className={`text-center font-bold ${
        data.selected ? 'bg-slate-400' : 'bg-relation-50'
      } border-l-relation-600 border-l-[3px] text-[13px] min-w-[200px]`}
      onMouseEnter={onMouseEnter}
      onMouseLeave={onMouseLeave}
    >
      <div>
        <span>
          {data.rightEntityHandleId && (
            <RightHandle id={data.rightEntityHandleId} type="source" point={direction} onDoubleClick={onChangeDirection} />
          )}
        </span>
        <div className="px-10">
          <span>{data?.name}</span>
          <span>
            <span>[</span>
            <input
              className={
                'bg-inherit text-center appearance-none mx-0.5 rounded-sm ' + (depth.min < 0 || depth.min > depth.max ? ' bg-red-400 ' : '')
              }
              style={{ maxWidth: calcWidth(depth.min) }}
              type="number"
              min={0}
              placeholder={'?'}
              value={depth.min}
              onChange={(e) => {
                setDepth({ ...depth, min: parseInt(e.target.value) });
              }}
              onBlur={(e) => {
                onNodeUpdated();
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onNodeUpdated();
                }
              }}
            ></input>
            <span>..</span>
            <input
              className={
                'bg-inherit text-center appearance-none mx-0.5 rounded-sm ' +
                (depth.max > 99 || depth.min > depth.max ? ' bg-red-400 ' : '')
              }
              style={{ maxWidth: calcWidth(depth.max) }}
              type="number"
              min={1}
              placeholder={'?'}
              value={depth.max}
              onChange={(e) => {
                setDepth({ ...depth, max: parseInt(e.target.value) });
              }}
              onBlur={(e) => {
                onNodeUpdated();
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onNodeUpdated();
                }
              }}
            ></input>
            <span>]</span>
          </span>
        </div>
        <span>
          {data.leftEntityHandleId && (
            <LeftHandle id={data.leftEntityHandleId} type="target" point={direction} onDoubleClick={onChangeDirection} />
          )}
        </span>
        {data?.attributes && (
          <PillDropdown
            node={node}
            attributes={data.attributes}
            attributeEdges={attributeEdges.map((edge) => edge?.attributes)}
            hovered={hovered}
            handleBeingDraggedIdx={handleBeingDragged}
            onHandleMouseDown={onHandleMouseDown}
          />
        )}
      </div>
    </div>
  );
});

RelationPill.displayName = 'RelationPill';
export default RelationPill;
